@@ -81,6 +81,10 @@ DO_USE_DOCKER = None
 # lambda executor instance
 LAMBDA_EXECUTOR = lambda_executors.AVAILABLE_EXECUTORS.get(config.LAMBDA_EXECUTOR, lambda_executors.DEFAULT_EXECUTOR)
 
+# IAM policy constants
+IAM_POLICY_VERSION = '2012-10-17'
+POLICY_NAME_PATTERN = 'lambda_policy_%s'
+
 # Marker name to indicate that a bucket represents the local file system. This is used for testing
 # Serverless applications where we mount the Lambda code directly into the container from the host OS.
 BUCKET_MARKER_LOCAL = '__local__'
@@ -248,7 +252,8 @@ def process_sqs_message(message_body, message_attributes, queue_name, region_nam
     try:
         queue_arn = aws_stack.sqs_queue_arn(queue_name, region_name=region_name)
         sources = get_event_sources(source_arn=queue_arn)
-        LOG.debug('Found %s source mappings for event from SQS queue %s' % (len(sources), queue_arn))
+        arns = [s.get('FunctionArn') for s in sources]
+        LOG.debug('Found %s source mappings for event from SQS queue %s: %s' % (len(arns), queue_arn, arns))
         source = next(iter(sources), None)
         if source:
             arn = source['FunctionArn']
@@ -394,6 +399,7 @@ def get_handler_file_from_name(handler_name, runtime=LAMBDA_DEFAULT_RUNTIME):
     elif runtime.startswith(LAMBDA_RUNTIME_CUSTOM_RUNTIME):
         file_ext = '.sh'
     else:
+        handler_name = handler_name.rpartition(delimiter)[0].replace(delimiter, os.path.sep)
         file_ext = '.py'
     return '%s%s' % (handler_name.split(delimiter)[0], file_ext)
 
@@ -559,12 +565,12 @@ def set_function_code(code, lambda_name, lambda_cwd=None):
             # running Lambdas locally (not in Docker), or (3) we're using remote Docker.
             # -> We do *not* want to raise an error if we're using local mount in non-remote Docker
             if not is_local_mount or not use_docker() or config.LAMBDA_REMOTE_DOCKER:
-                file_list = run('cd %s; du -d 3 .' % lambda_cwd)
+                file_list = run('cd "%s"; du -d 3 .' % lambda_cwd)
                 config_debug = ('Config for local mount, docker, remote: "%s", "%s", "%s"' %
                     (is_local_mount, use_docker(), config.LAMBDA_REMOTE_DOCKER))
                 LOG.debug('Lambda archive content:\n%s' % file_list)
                 raise ClientError(error_response(
-                    'Unable to find handler script in Lambda archive. %s' % config_debug,
+                    'Unable to find handler script (%s) in Lambda archive. %s' % (main_file, config_debug),
                     400, error_type='ValidationError'))
 
         if runtime.startswith('python') and not use_docker():
@@ -594,19 +600,23 @@ def do_list_functions():
         func_name = f_arn.split(':function:')[-1]
         arn = func_arn(func_name)
         func_details = arn_to_lambda.get(arn)
+        if not func_details:
+            # this can happen if we're accessing Lambdas from a different region (ARN mismatch)
+            continue
         funcs.append(format_func_details(func_details))
     return funcs
 
 
 def format_func_details(func_details, version=None, always_add_version=False):
     version = version or '$LATEST'
+    func_version = func_details.get_version(version)
     result = {
-        'CodeSha256': func_details.get_version(version).get('CodeSha256'),
+        'CodeSha256': func_version.get('CodeSha256'),
         'Role': func_details.role,
         'Version': version,
         'FunctionArn': func_details.arn(),
         'FunctionName': func_details.name(),
-        'CodeSize': func_details.get_version(version).get('CodeSize'),
+        'CodeSize': func_version.get('CodeSize'),
         'Handler': func_details.handler,
         'Runtime': func_details.runtime,
         'Timeout': func_details.timeout,
@@ -614,7 +624,7 @@ def format_func_details(func_details, version=None, always_add_version=False):
         'MemorySize': func_details.memory_size,
         'LastModified': func_details.last_modified,
         'TracingConfig': {'Mode': 'PassThrough'},
-        'RevisionId': func_details.get_version(version).get('RevisionId')
+        'RevisionId': func_version.get('RevisionId')
     }
     if func_details.envvars:
         result['Environment'] = {
@@ -647,6 +657,25 @@ def forward_to_fallback_url(func_arn, data):
     raise ClientError('Unexpected value for LAMBDA_FALLBACK_URL: %s' % config.LAMBDA_FALLBACK_URL)
 
 
+def get_lambda_policy(function):
+    iam_client = aws_stack.connect_to_service('iam')
+    policies = iam_client.list_policies(Scope='Local', MaxItems=500)['Policies']
+    docs = []
+    for p in policies:
+        # !TODO: Cache policy documents instead of running N+1 API calls here!
+        versions = iam_client.list_policy_versions(PolicyArn=p['Arn'])['Versions']
+        default_version = [v for v in versions if v.get('IsDefaultVersion')]
+        versions = default_version or versions
+        doc = versions[0]['Document']
+        doc = doc if isinstance(doc, dict) else json.loads(doc)
+        if not isinstance(doc['Statement'], list):
+            doc['Statement'] = [doc['Statement']]
+        doc['PolicyArn'] = p['Arn']
+        docs.append(doc)
+    policy = [d for d in docs if d['Statement'][0]['Resource'] == func_arn(function)]
+    return (policy or [None])[0]
+
+
 def not_found_error(ref=None, msg=None):
     if not msg:
         msg = 'The resource you requested does not exist.'
@@ -869,6 +898,54 @@ def update_function_configuration(function):
     return jsonify(result)
 
 
+@app.route('%s/functions/<function>/policy' % PATH_ROOT, methods=['POST'])
+def add_permission(function):
+    data = json.loads(to_str(request.data))
+    iam_client = aws_stack.connect_to_service('iam')
+    sid = short_uid()
+    policy = {
+        'Version': IAM_POLICY_VERSION,
+        'Id': 'LambdaFuncAccess-%s' % sid,
+        'Statement': [{
+            'Sid': sid,
+            'Effect': 'Allow',
+            # TODO: 'Principal' in policies not yet supported in upstream moto
+            # 'Principal': data.get('Principal') or {'AWS': TEST_AWS_ACCOUNT_ID},
+            'Action': data.get('Action'),
+            'Resource': func_arn(function)
+        }]
+    }
+    iam_client.create_policy(PolicyName=POLICY_NAME_PATTERN % function,
+        PolicyDocument=json.dumps(policy), Description='Policy for Lambda function "%s"' % function)
+    result = {'Statement': sid}
+    return jsonify(result)
+
+
+@app.route('%s/functions/<function>/policy/<statement>' % PATH_ROOT, methods=['DELETE'])
+def remove_permission(function, statement):
+    qualifier = request.args.get('Qualifier')
+    iam_client = aws_stack.connect_to_service('iam')
+    policy = get_lambda_policy(function)
+    if not policy:
+        return error_response('Unable to find policy for Lambda function "%s"' % function,
+            404, error_type='ResourceNotFoundException')
+    iam_client.delete_policy(PolicyArn=policy['PolicyArn'])
+    result = {
+        'FunctionName': function,
+        'Qualifier': qualifier,
+        'StatementId': policy['Statement'][0]['Sid'],
+    }
+    return jsonify(result)
+
+
+@app.route('%s/functions/<function>/policy' % PATH_ROOT, methods=['GET'])
+def get_policy(function):
+    policy = get_lambda_policy(function)
+    if not policy:
+        return jsonify({}), 404
+    return jsonify({'Policy': policy})
+
+
 @app.route('%s/functions/<function>/invocations' % PATH_ROOT, methods=['POST'])
 def invoke_function(function):
     """ Invoke an existing function
