@@ -5,12 +5,12 @@ import time
 import logging
 import threading
 import subprocess
+import six
 from multiprocessing import Process, Queue
 try:
     from shlex import quote as cmd_quote
 except ImportError:
-    # for Python 2.7
-    from pipes import quote as cmd_quote
+    from pipes import quote as cmd_quote  # for Python 2.7
 from localstack import config
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import (
@@ -102,7 +102,14 @@ class LambdaExecutor(object):
         log_groups = logs_client.describe_log_groups()['logGroups']
         log_groups = [lg['logGroupName'] for lg in log_groups]
         if log_group_name not in log_groups:
-            logs_client.create_log_group(logGroupName=log_group_name)
+            try:
+                logs_client.create_log_group(logGroupName=log_group_name)
+            except Exception as e:
+                if 'ResourceAlreadyExistsException' in str(e):
+                    # this can happen in certain cases, possibly due to a race condition
+                    pass
+                else:
+                    raise e
 
         # create a new log stream for this lambda invocation
         logs_client.create_log_stream(logGroupName=log_group_name, logStreamName=log_stream_name)
@@ -132,13 +139,16 @@ class LambdaExecutor(object):
         process = run(cmd, asynchronous=True, stderr=subprocess.PIPE, outfile=subprocess.PIPE, env_vars=env_vars,
                       stdin=True)
         result, log_output = process.communicate(input=event)
-        result = to_str(result).strip()
+        try:
+            result = to_str(result).strip()
+        except Exception:
+            pass
         log_output = to_str(log_output).strip()
         return_code = process.returncode
         # Note: The user's code may have been logging to stderr, in which case the logs
         # will be part of the "result" variable here. Hence, make sure that we extract
         # only the *last* line of "result" and consider anything above that as log output.
-        if '\n' in result:
+        if isinstance(result, six.string_types) and '\n' in result:
             additional_logs, _, result = result.rpartition('\n')
             log_output += '\n%s' % additional_logs
 
@@ -271,7 +281,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         copy_command = ''
         docker_cmd = self._docker_cmd()
         event_file = os.path.join(lambda_cwd, LAMBDA_EVENT_FILE)
-        if not has_been_invoked_before:
+        if not has_been_invoked_before and config.LAMBDA_REMOTE_DOCKER:
             # if this is the first invocation: copy the entire folder into the container
             copy_command = '%s cp "%s/." "%s:/var/task";' % (docker_cmd, lambda_cwd, container_info.name)
         elif os.path.exists(event_file):
@@ -318,6 +328,8 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             status = self.get_docker_container_status(func_arn)
             LOG.debug('Priming docker container (status "%s"): %s' % (status, container_name))
 
+            lambda_container_registry = config.LAMBDA_CONTAINER_REGISTRY
+
             # Container is not running or doesn't exist.
             if status < 1:
                 # Make sure the container does not exist in any form/state.
@@ -328,6 +340,10 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                 network = config.LAMBDA_DOCKER_NETWORK
                 network_str = '--network="%s"' % network if network else ''
 
+                mount_volume = not config.LAMBDA_REMOTE_DOCKER
+                lambda_cwd_on_host = Util.get_host_path_for_path_in_docker(lambda_cwd)
+                mount_volume_str = '-v "%s":/var/task' % lambda_cwd_on_host if mount_volume else ''
+
                 # Create and start the container
                 LOG.debug('Creating container: %s' % container_name)
                 cmd = (
@@ -335,24 +351,27 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                     ' --rm'
                     ' --name "%s"'
                     ' --entrypoint /bin/bash'  # Load bash when it starts.
+                    ' %s'
                     ' --interactive'  # Keeps the container running bash.
                     ' -e AWS_LAMBDA_EVENT_BODY="$AWS_LAMBDA_EVENT_BODY"'
                     ' -e HOSTNAME="$HOSTNAME"'
                     ' -e LOCALSTACK_HOSTNAME="$LOCALSTACK_HOSTNAME"'
                     '  %s'  # env_vars
                     '  %s'  # network
-                    ' lambci/lambda:%s'
-                ) % (docker_cmd, container_name, env_vars_str, network_str, runtime)
+                    ' %s:%s'
+                ) % (docker_cmd, container_name, mount_volume_str, env_vars_str, network_str,
+                     lambda_container_registry, runtime)
                 LOG.debug(cmd)
                 run(cmd)
 
-                LOG.debug('Copying files to container "%s" from "%s".' % (container_name, lambda_cwd))
-                cmd = (
-                    '%s cp'
-                    ' "%s/." "%s:/var/task"'
-                ) % (docker_cmd, lambda_cwd, container_name)
-                LOG.debug(cmd)
-                run(cmd)
+                if not mount_volume:
+                    LOG.debug('Copying files to container "%s" from "%s".' % (container_name, lambda_cwd))
+                    cmd = (
+                        '%s cp'
+                        ' "%s/." "%s:/var/task"'
+                    ) % (docker_cmd, lambda_cwd, container_name)
+                    LOG.debug(cmd)
+                    run(cmd)
 
                 LOG.debug('Starting container: %s' % container_name)
                 cmd = '%s start %s' % (docker_cmd, container_name)
@@ -362,12 +381,12 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                 time.sleep(1)
 
             # Get the entry point for the image.
-            LOG.debug('Getting the entrypoint for image: lambci/lambda:%s' % runtime)
+            LOG.debug('Getting the entrypoint for image: %s:%s' % (lambda_container_registry, runtime))
             cmd = (
                 '%s image inspect'
                 ' --format="{{ .ContainerConfig.Entrypoint }}"'
-                ' lambci/lambda:%s'
-            ) % (docker_cmd, runtime)
+                ' %s:%s'
+            ) % (docker_cmd, lambda_container_registry, runtime)
 
             LOG.debug(cmd)
             run_result = run(cmd)
@@ -567,6 +586,7 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
         network = config.LAMBDA_DOCKER_NETWORK
         network_str = '--network="%s"' % network if network else ''
         docker_cmd = self._docker_cmd()
+        lambda_container_registry = config.LAMBDA_CONTAINER_REGISTRY
 
         if config.LAMBDA_REMOTE_DOCKER:
             cmd = (
@@ -576,29 +596,27 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
                 ' %s'
                 ' %s'  # network
                 ' --rm'
-                ' "lambci/lambda:%s" %s'
+                ' "%s:%s" %s'
                 ')";'
                 '%s cp "%s/." "$CONTAINER_ID:/var/task"; '
                 '%s start -ai "$CONTAINER_ID";'
-            ) % (docker_cmd, entrypoint, debug_docker_java_port, env_vars_string, network_str, runtime, command,
+            ) % (docker_cmd, entrypoint, debug_docker_java_port, env_vars_string, network_str,
+                 lambda_container_registry, runtime, command,
                  docker_cmd, lambda_cwd,
                  docker_cmd)
         else:
-            lambda_cwd_on_host = self.get_host_path_for_path_in_docker(lambda_cwd)
+            lambda_cwd_on_host = Util.get_host_path_for_path_in_docker(lambda_cwd)
             cmd = (
                 '%s run -i'
                 ' %s -v "%s":/var/task'
                 ' %s'
                 ' %s'  # network
                 ' --rm'
-                ' "lambci/lambda:%s" %s'
-            ) % (docker_cmd, entrypoint, lambda_cwd_on_host, env_vars_string, network_str, runtime, command)
+                ' "%s:%s" %s'
+            ) % (docker_cmd, entrypoint, lambda_cwd_on_host, env_vars_string, network_str,
+                 lambda_container_registry, runtime, command)
         return cmd
 
-    def get_host_path_for_path_in_docker(self, path):
-        return re.sub(r'^%s/(.*)$' % config.TMP_FOLDER,
-                    r'%s/\1' % config.HOST_TMP_FOLDER, path)
-
 
 class LambdaExecutorLocal(LambdaExecutor):
 
@@ -656,6 +674,11 @@ class Util:
             opts = opts.replace('_debug_port_', ('%s' % cls.debug_java_port))
         return opts
 
+    @classmethod
+    def get_host_path_for_path_in_docker(cls, path):
+        return re.sub(r'^%s/(.*)$' % config.TMP_FOLDER,
+                      r'%s/\1' % config.HOST_TMP_FOLDER, path)
+
 
 # --------------
 # GLOBAL STATE
