@@ -1,4 +1,5 @@
 import sys
+import json
 import logging
 import traceback
 import six
@@ -16,7 +17,7 @@ from boto.cloudformation.stack import Output
 from moto.cloudformation.exceptions import ValidationError, UnformattedGetAttTemplateException
 from localstack import config
 from localstack.constants import DEFAULT_PORT_CLOUDFORMATION_BACKEND, TEST_AWS_ACCOUNT_ID, MOTO_ACCOUNT_ID
-from localstack.utils.aws import aws_stack
+from localstack.utils.aws import aws_stack, aws_responses
 from localstack.utils.common import FuncThread, short_uid, recurse_object, clone, json_safe
 from localstack.stepfunctions import models as sfn_models
 from localstack.services.infra import (
@@ -36,7 +37,14 @@ RUN_SERVER_IN_PROCESS = False
 
 # map of additional model classes
 MODEL_MAP = {
-    'AWS::StepFunctions::Activity': service_models.StepFunctionsActivity
+    'AWS::StepFunctions::Activity': service_models.StepFunctionsActivity,
+    'AWS::SNS::Subscription': service_models.SNSSubscription,
+    'AWS::ApiGateway::GatewayResponse': service_models.GatewayResponse,
+    'AWS::ApiGateway::Deployment': apigw_models.Deployment,
+    'AWS::ApiGateway::Method': apigw_models.Method,
+    'AWS::ApiGateway::Resource': apigw_models.Resource,
+    'AWS::ApiGateway::RestApi': apigw_models.RestAPI,
+    'AWS::StepFunctions::StateMachine': sfn_models.StateMachine
 }
 
 
@@ -73,7 +81,7 @@ def get_entity_id(entity, resource_json=None):
         return entity.physical_resource_id
     # check ID attribute candidates
     types_with_ref_as_id_or_name = (apigw_models.RestAPI, apigw_models.Resource)
-    attr_candidates = ['function_arn', 'Arn', 'id', 'name', 'Id', 'Name']
+    attr_candidates = ['function_arn', 'Arn', 'Ref', 'id', 'Id', 'name', 'Name']
     for attr in attr_candidates:
         if hasattr(entity, attr):
             if attr in ['id', 'name'] and not isinstance(entity, types_with_ref_as_id_or_name):
@@ -107,6 +115,22 @@ def convert_objs_to_ids(resource_json):
     return recurse_object(resource_json, fix_ids)
 
 
+def update_physical_resource_id(resource):
+    phys_res_id = getattr(resource, 'physical_resource_id', None)
+    if not phys_res_id:
+        if isinstance(resource, lambda_models.LambdaFunction):
+            func_arn = aws_stack.lambda_function_arn(resource.function_name)
+            resource.function_arn = resource.physical_resource_id = func_arn
+        elif isinstance(resource, sfn_models.StateMachine):
+            sm_arn = aws_stack.state_machine_arn(resource.name)
+            resource.physical_resource_id = sm_arn
+        elif isinstance(resource, service_models.StepFunctionsActivity):
+            act_arn = aws_stack.stepfunctions_activity_arn(resource.params.get('Name'))
+            resource.physical_resource_id = act_arn
+        else:
+            LOG.warning('Unable to determine physical_resource_id for resource %s' % type(resource))
+
+
 def apply_patches():
     """ Apply patches to make LocalStack seamlessly interact with the moto backend.
         TODO: Eventually, these patches should be contributed to the upstream repo! """
@@ -152,7 +176,16 @@ def apply_patches():
                       (logical_id, e, traceback.format_exc()))
             raise
 
-    def _parse_and_create_resource(logical_id, resource_json, resources_map, region_name):
+    def parse_and_update_resource(logical_id, resource_json, resources_map, region_name):
+        try:
+            return _parse_and_create_resource(logical_id,
+                resource_json, resources_map, region_name, update=True)
+        except Exception as e:
+            LOG.error('Unable to parse and update resource "%s": %s %s' %
+                      (logical_id, e, traceback.format_exc()))
+            raise
+
+    def _parse_and_create_resource(logical_id, resource_json, resources_map, region_name, update=False):
         stack_name = resources_map.get('AWS::StackName')
         resource_hash_key = (stack_name, logical_id)
 
@@ -175,36 +208,55 @@ def apply_patches():
 
         # check if this resource already exists in the resource map
         resource = resources_map._parsed_resources.get(logical_id)
+        if resource and not update:
+            return resource
 
         # check whether this resource needs to be deployed
         resource_wrapped = {logical_id: resource_json}
         should_be_created = template_deployer.should_be_deployed(logical_id, resource_wrapped, stack_name)
+
+        # fix resource ARNs, make sure to convert account IDs 000000000000 to 123456789012
+        resource_json_arns_fixed = clone(json_safe(convert_objs_to_ids(resource_json)))
+        set_moto_account_ids(resource_json_arns_fixed)
+
+        # create resource definition and store CloudFormation metadata in moto
+        if resource or update:
+            parse_and_update_resource_orig(logical_id,
+                resource_json_arns_fixed, resources_map, region_name)
+        elif not resource:
+            try:
+                resource = parse_and_create_resource_orig(logical_id,
+                    resource_json_arns_fixed, resources_map, region_name)
+            except Exception as e:
+                if should_be_created:
+                    raise
+                else:
+                    LOG.info('Error on moto CF resource creation. Ignoring, as should_be_created=%s: %s' %
+                             (should_be_created, e))
+
+        # Fix for moto which sometimes hard-codes region name as 'us-east-1'
+        if hasattr(resource, 'region_name') and resource.region_name != region_name:
+            LOG.debug('Updating incorrect region from %s to %s' % (resource.region_name, region_name))
+            resource.region_name = region_name
+
+        # check whether this resource needs to be deployed
+        is_updateable = False
         if not should_be_created:
             # This resource is either not deployable or already exists. Check if it can be updated
-            if not template_deployer.is_updateable(logical_id, resource_wrapped, stack_name):
-                LOG.debug('Resource %s need not be deployed: %s' % (logical_id, resource_json))
-                if resource:
-                    return resource
-
-        if not resource:
-            # fix resource ARNs, make sure to convert account IDs 000000000000 to 123456789012
-            resource_json_arns_fixed = clone(json_safe(convert_objs_to_ids(resource_json)))
-            set_moto_account_ids(resource_json_arns_fixed)
-            # create resource definition and store CloudFormation metadata in moto
-            resource = parse_and_create_resource_orig(logical_id,
-                resource_json_arns_fixed, resources_map, region_name)
-            # Fix for moto which sometimes hard-codes region name as 'us-east-1'
-            if hasattr(resource, 'region_name') and resource.region_name != region_name:
-                LOG.debug('Updating incorrect region from %s to %s' % (resource.region_name, region_name))
-                resource.region_name = region_name
+            is_updateable = template_deployer.is_updateable(logical_id, resource_wrapped, stack_name)
+            if not update or not is_updateable:
+                LOG.debug('Resource %s need not be deployed: %s %s' % (logical_id, resource_json, bool(resource)))
+                # Return if this resource already exists and can/need not be updated
+                return resource
 
         # Apply some fixes/patches to the resource names, then deploy resource in LocalStack
         update_resource_name(resource, resource_json)
-        LOG.debug('Deploying CloudFormation resource: %s' % resource_json)
+        LOG.debug('Deploying CloudFormation resource (update=%s, exists=%s, updateable=%s): %s' %
+                  (update, not should_be_created, is_updateable, resource_json))
 
         try:
             CURRENTLY_UPDATING_RESOURCES[resource_hash_key] = True
-            deploy_func = template_deployer.deploy_resource if should_be_created else template_deployer.update_resource
+            deploy_func = template_deployer.update_resource if update else template_deployer.deploy_resource
             result = deploy_func(logical_id, resource_wrapped, stack_name=stack_name)
         finally:
             CURRENTLY_UPDATING_RESOURCES[resource_hash_key] = False
@@ -215,6 +267,8 @@ def apply_patches():
 
         def find_id(resource):
             """ Find ID of the given resource. """
+            if not resource:
+                return
             for id_attr in ('Id', 'id', 'ResourceId', 'RestApiId', 'DeploymentId'):
                 if id_attr in resource:
                     return resource[id_attr]
@@ -288,23 +342,10 @@ def apply_patches():
         else:
             LOG.warning('Unexpected resource type when updating ID: %s' % type(resource))
 
-    def update_physical_resource_id(resource):
-        phys_res_id = getattr(resource, 'physical_resource_id', None)
-        if not phys_res_id:
-            if isinstance(resource, lambda_models.LambdaFunction):
-                func_arn = aws_stack.lambda_function_arn(resource.function_name)
-                resource.function_arn = resource.physical_resource_id = func_arn
-            elif isinstance(resource, sfn_models.StateMachine):
-                sm_arn = aws_stack.state_machine_arn(resource.name)
-                resource.physical_resource_id = sm_arn
-            elif isinstance(resource, service_models.StepFunctionsActivity):
-                act_arn = aws_stack.stepfunctions_activity_arn(resource.params.get('Name'))
-                resource.physical_resource_id = act_arn
-            else:
-                LOG.warning('Unable to determine physical_resource_id for resource %s' % type(resource))
-
     parse_and_create_resource_orig = parsing.parse_and_create_resource
     parsing.parse_and_create_resource = parse_and_create_resource
+    parse_and_update_resource_orig = parsing.parse_and_update_resource
+    parsing.parse_and_update_resource = parse_and_update_resource
 
     # Patch CloudFormation parse_output(..) method to fix a bug in moto
 
@@ -350,7 +391,7 @@ def apply_patches():
     # Patch SQS get_cfn_attribute(..) method in moto
 
     def SQS_Queue_get_cfn_attribute(self, attribute_name):
-        if attribute_name == 'Arn':
+        if attribute_name in ['Arn', 'QueueArn']:
             return aws_stack.sqs_queue_arn(queue_name=self.name)
         return SQS_Queue_get_cfn_attribute_orig(self, attribute_name)
 
@@ -413,6 +454,17 @@ def apply_patches():
     IAM_Role_get_cfn_attribute_orig = iam_models.Role.get_cfn_attribute
     iam_models.Role.get_cfn_attribute = IAM_Role_get_cfn_attribute
 
+    # Patch SNS Topic get_cfn_attribute(..) method in moto
+
+    def SNS_Topic_get_cfn_attribute(self, attribute_name):
+        result = SNS_Topic_get_cfn_attribute(self, attribute_name)
+        if attribute_name.lower() in ['arn', 'topicarn']:
+            result = aws_stack.fix_account_id_in_arns(result)
+        return result
+
+    IAM_Role_get_cfn_attribute_orig = iam_models.Role.get_cfn_attribute
+    iam_models.Role.get_cfn_attribute = IAM_Role_get_cfn_attribute
+
     # Patch LambdaFunction create_from_cloudformation_json(..) method in moto
 
     @classmethod
@@ -423,13 +475,51 @@ def apply_patches():
     Lambda_create_from_cloudformation_json_orig = lambda_models.LambdaFunction.create_from_cloudformation_json
     lambda_models.LambdaFunction.create_from_cloudformation_json = Lambda_create_from_cloudformation_json
 
-    # add CloudWatch types
+    # Patch EventSourceMapping create_from_cloudformation_json(..) method in moto
 
-    parsing.MODEL_MAP['AWS::ApiGateway::Deployment'] = apigw_models.Deployment
-    parsing.MODEL_MAP['AWS::ApiGateway::Method'] = apigw_models.Method
-    parsing.MODEL_MAP['AWS::ApiGateway::Resource'] = apigw_models.Resource
-    parsing.MODEL_MAP['AWS::ApiGateway::RestApi'] = apigw_models.RestAPI
-    parsing.MODEL_MAP['AWS::StepFunctions::StateMachine'] = sfn_models.StateMachine
+    @classmethod
+    def Mapping_create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
+        props = cloudformation_json.get('Properties', {})
+        func_name = props.get('FunctionName') or ''
+        if ':lambda:' in func_name:
+            props['FunctionName'] = aws_stack.lambda_function_name(func_name)
+        return Mapping_create_from_cloudformation_json_orig(resource_name, cloudformation_json, region_name)
+
+    Mapping_create_from_cloudformation_json_orig = lambda_models.EventSourceMapping.create_from_cloudformation_json
+    lambda_models.EventSourceMapping.create_from_cloudformation_json = Mapping_create_from_cloudformation_json
+
+    # Patch LambdaFunction update_from_cloudformation_json(..) method in moto
+
+    @classmethod
+    def Lambda_update_from_cloudformation_json(cls,
+            original_resource, new_resource_name, cloudformation_json, region_name):
+        resource_name = cloudformation_json.get('Properties', {}).get('FunctionName') or new_resource_name
+        return Lambda_create_from_cloudformation_json_orig(resource_name, cloudformation_json, region_name)
+
+    if not hasattr(lambda_models.LambdaFunction, 'update_from_cloudformation_json'):
+        lambda_models.LambdaFunction.update_from_cloudformation_json = Lambda_update_from_cloudformation_json
+
+    # patch ApiGateway Deployment
+
+    def depl_delete_from_cloudformation_json(
+            resource_name, resource_json, region_name):
+        properties = resource_json['Properties']
+        LOG.info('TODO: apigateway.Deployment.delete_from_cloudformation_json %s' % properties)
+
+    if not hasattr(apigw_models.Deployment, 'delete_from_cloudformation_json'):
+        apigw_models.Deployment.delete_from_cloudformation_json = depl_delete_from_cloudformation_json
+
+    # patch Lambda Version
+
+    def vers_delete_from_cloudformation_json(
+            resource_name, resource_json, region_name):
+        properties = resource_json['Properties']
+        LOG.info('TODO: apigateway.Deployment.delete_from_cloudformation_json %s' % properties)
+
+    if not hasattr(lambda_models.LambdaVersion, 'delete_from_cloudformation_json'):
+        lambda_models.LambdaVersion.delete_from_cloudformation_json = vers_delete_from_cloudformation_json
+
+    # add CloudFormation types
 
     @classmethod
     def RestAPI_create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
@@ -494,6 +584,14 @@ def apply_patches():
         stack_name = self._get_param('StackName')
         stack = self.cloudformation_backend.get_stack(stack_name)
         logical_resource_id = self._get_param('LogicalResourceId')
+        if not stack:
+            msg = ('Unable to find CloudFormation stack "%s" in region %s' %
+                   (stack_name, aws_stack.get_region()))
+            if aws_stack.get_region() != self.region:
+                msg = '%s/%s' % (msg, self.region)
+            LOG.warning(msg)
+            response = aws_responses.flask_error_response(msg, code=404, error_type='ResourceNotFoundException')
+            return 404, response.headers, response.data
 
         for stack_resource in stack.stack_resources:
             # Note: Line below has been patched
@@ -514,7 +612,6 @@ def apply_patches():
 def inject_stats_endpoint():
     """ Inject a simple /_stats endpoint into the moto server backend Web app. """
     # TODO: move this utility method to a shared file and enable it for all API endpoints
-    import json
     from moto import server as moto_server
 
     def _get_stats():
