@@ -1,4 +1,4 @@
-class StepFunctionsActivity(object):
+class BaseModel(object):
 
     def __init__(self, **params):
         self.params = params
@@ -6,4 +6,16 @@ class StepFunctionsActivity(object):
     @classmethod
     def create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
         props = cloudformation_json['Properties']
-        return StepFunctionsActivity(**props)
+        return cls(**props)
+
+
+class StepFunctionsActivity(BaseModel):
+    pass
+
+
+class SNSSubscription(BaseModel):
+    pass
+
+
+class GatewayResponse(BaseModel):
+    pass
