@@ -18,7 +18,7 @@ from six.moves.socketserver import ThreadingMixIn
 from six.moves.urllib.parse import urlparse
 from six.moves.BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
 from localstack.config import TMP_FOLDER, USE_SSL, EXTRA_CORS_ALLOWED_HEADERS, EXTRA_CORS_EXPOSE_HEADERS
-from localstack.constants import ENV_INTERNAL_TEST_RUN
+from localstack.constants import ENV_INTERNAL_TEST_RUN, APPLICATION_JSON
 from localstack.utils.common import FuncThread, generate_ssl_cert, to_bytes
 
 QUIET = False
@@ -29,7 +29,7 @@ SERVER_CERT_PEM_FILE = '%s/server.test.pem' % (TMP_FOLDER)
 
 CORS_ALLOWED_HEADERS = ['authorization', 'content-type', 'content-md5', 'cache-control',
     'x-amz-content-sha256', 'x-amz-date', 'x-amz-security-token', 'x-amz-user-agent',
-    'x-amz-target', 'x-amz-acl', 'x-amz-version-id', 'x-localstack-target']
+    'x-amz-target', 'x-amz-acl', 'x-amz-version-id', 'x-localstack-target', 'x-amz-tagging']
 if EXTRA_CORS_ALLOWED_HEADERS:
     CORS_ALLOWED_HEADERS += EXTRA_CORS_ALLOWED_HEADERS.split(',')
 
@@ -67,7 +67,7 @@ class ProxyListener(object):
         """
         return True
 
-    def return_response(self, method, path, data, headers, response):
+    def return_response(self, method, path, data, headers, response, request_handler=None):
         """ This interceptor method is called by the proxy when returning a response
             (*after* having forwarded the request and received a response from the backend
             service). It receives details of the incoming request as well as the response
@@ -244,6 +244,7 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
                 if isinstance(listener_result, dict):
                     response = Response()
                     response._content = json.dumps(listener_result)
+                    response.headers['Content-Type'] = APPLICATION_JSON
                     response.status_code = 200
                     break
                 elif isinstance(listener_result, Request):
@@ -337,7 +338,8 @@ class GenericProxyHandler(BaseHTTPRequestHandler):
                 LOG.warning('Unable to flush write file: %s' % e)
 
     def _send_cors_headers(self, response=None):
-        headers = response and response.headers or {}
+        # Note: Use "response is not None" here instead of "not response"!
+        headers = response is not None and response.headers or {}
         if 'Access-Control-Allow-Origin' not in headers:
             self.send_header('Access-Control-Allow-Origin', '*')
         if 'Access-Control-Allow-Methods' not in headers:
