@@ -15,6 +15,11 @@ def flask_error_response(msg, code=500, error_type='InternalFailure'):
     return Response(json.dumps(result), status=code, headers=headers)
 
 
+def requests_error_response(msg, code=500, error_type='InternalFailure'):
+    response = flask_error_response(msg, code=code, error_type=error_type)
+    return flask_to_requests_response(response)
+
+
 def requests_response(content, status_code=200, headers={}):
     resp = RequestsResponse()
     content = json.dumps(content) if isinstance(content, dict) else content
@@ -22,3 +27,11 @@ def requests_response(content, status_code=200, headers={}):
     resp.status_code = status_code
     resp.headers = headers
     return resp
+
+
+def flask_to_requests_response(r):
+    return requests_response(r.data, status_code=r.status_code, headers=r.headers)
+
+
+def requests_to_flask_response(r):
+    return Response(r.content, status=r.status_code, headers=dict(r.headers))
