@@ -8,12 +8,10 @@ import logging
 import six
 from localstack import config
 from localstack.constants import (
-    REGION_LOCAL, LOCALHOST, MOTO_ACCOUNT_ID,
-    ENV_DEV, APPLICATION_AMZ_JSON_1_1, APPLICATION_AMZ_JSON_1_0,
-    APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID)
+    REGION_LOCAL, LOCALHOST, MOTO_ACCOUNT_ID, ENV_DEV, APPLICATION_AMZ_JSON_1_1,
+    APPLICATION_AMZ_JSON_1_0, APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID)
 from localstack.utils.common import (
-    run_safe, to_str, is_string, is_string_or_bytes, make_http_request,
-    is_port_open, get_service_protocol)
+    run_safe, to_str, is_string, is_string_or_bytes, make_http_request, is_port_open, get_service_protocol)
 from localstack.utils.aws.aws_models import KinesisStream
 
 # AWS environment variable names
@@ -183,8 +181,9 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
     """
     Generic method to obtain an AWS service client using boto3, based on environment, region, or custom endpoint_url.
     """
+    region_name = region_name or get_region()
     env = get_environment(env, region_name=region_name)
-    region = env.region if env.region != REGION_LOCAL else get_region()
+    region = env.region if env.region != REGION_LOCAL else region_name
     key_elements = [service_name, client, env, region, endpoint_url, config]
     cache_key = '/'.join([str(k) for k in key_elements])
     if cache_key not in BOTO_CLIENTS_CACHE:
@@ -348,14 +347,25 @@ def dynamodb_stream_arn(table_name, latest_stream_label, account_id=None):
 
 
 def lambda_function_arn(function_name, account_id=None):
+    return lambda_function_or_layer_arn('function', function_name, account_id=account_id)
+
+
+def lambda_layer_arn(layer_name, version=None, account_id=None):
+    return lambda_function_or_layer_arn('layer', layer_name, version=None, account_id=account_id)
+
+
+def lambda_function_or_layer_arn(type, entity_name, version=None, account_id=None):
     pattern = 'arn:aws:lambda:.*:.*:(function|layer):.*'
-    if re.match(pattern, function_name):
-        return function_name
-    if ':' in function_name:
-        raise Exception('Lambda function name should not contain a colon ":"')
+    if re.match(pattern, entity_name):
+        return entity_name
+    if ':' in entity_name:
+        raise Exception('Lambda %s name should not contain a colon ":": %s' % (type, entity_name))
     account_id = get_account_id(account_id)
-    pattern = re.sub(r'\([^\|]+\|.+\)', 'function', pattern)
-    return pattern.replace('.*', '%s') % (get_region(), account_id, function_name)
+    pattern = re.sub(r'\([^\|]+\|.+\)', type, pattern)
+    result = pattern.replace('.*', '%s') % (get_region(), account_id, entity_name)
+    if version:
+        result = '%s:%s' (result, version)
+    return result
 
 
 def lambda_function_name(name_or_arn):
