@@ -3,6 +3,7 @@ import json
 import yaml
 import logging
 import traceback
+import moto.cloudformation.utils
 from six import iteritems
 from six import string_types
 from localstack.utils import common
@@ -33,6 +34,10 @@ def get_bucket_location_config(**kwargs):
     return {'LocationConstraint': aws_stack.get_region()}
 
 
+def lambda_get_params():
+    return lambda params, **kwargs: params
+
+
 # maps resource types to functions and parameters for creation
 RESOURCE_TO_FUNCTION = {
     'S3::Bucket': {
@@ -94,7 +99,9 @@ RESOURCE_TO_FUNCTION = {
         'create': {
             'function': 'publish_version',
             'parameters': {
-                # TODO
+                'FunctionName': 'FunctionName',
+                'CodeSha256': 'CodeSha256',
+                'Description': 'Description'
             }
         }
     },
@@ -133,6 +140,25 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
+    'Events::Rule': {
+        'create': [{
+            'function': 'put_rule',
+            'parameters': {
+                'Name': PLACEHOLDER_RESOURCE_NAME,
+                'ScheduleExpression': 'ScheduleExpression',
+                'EventPattern': 'EventPattern',
+                'State': 'State',
+                'Description': 'Description'
+            }
+        }, {
+            'function': 'put_targets',
+            'parameters': {
+                'Rule': PLACEHOLDER_RESOURCE_NAME,
+                'EventBusName': 'EventBusName',
+                'Targets': 'Targets'
+            }
+        }]
+    },
     'IAM::Role': {
         # TODO implement
     },
@@ -180,6 +206,18 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
+    'ApiGateway::GatewayResponse': {
+        'create': {
+            'function': 'put_gateway_response',
+            'parameters': {
+                'restApiId': 'RestApiId',
+                'responseType': 'ResponseType',
+                'statusCode': 'StatusCode',
+                'responseParameters': 'ResponseParameters',
+                'responseTemplates': 'ResponseTemplates'
+            }
+        }
+    },
     'Kinesis::Stream': {
         'create': {
             'function': 'create_stream',
@@ -210,6 +248,16 @@ RESOURCE_TO_FUNCTION = {
                 'tags': 'Tags'
             }
         }
+    },
+    'SNS::Subscription': {
+        'create': {
+            'function': 'subscribe',
+            'parameters': {
+                'TopicArn': 'TopicArn',
+                'Protocol': 'Protocol',
+                'Endpoint': 'Endpoint'
+            }
+        }
     }
 }
 
@@ -242,7 +290,11 @@ def parse_template(template):
     try:
         return json.loads(template)
     except Exception:
-        return yaml.safe_load(template)
+        yaml.add_multi_constructor('', moto.cloudformation.utils.yaml_tag_constructor)
+        try:
+            return yaml.safe_load(template)
+        except Exception:
+            return yaml.load(template, Loader=yaml.Loader)
 
 
 def template_to_json(template):
@@ -266,7 +318,6 @@ def get_service_name(resource):
     if res_type.endswith('Cognito::UserPool'):
         return 'cognito-idp'
     if parts[-2] == 'Cognito':
-        # TODO add mappings for "cognito-identity"
         return 'cognito-idp'
     return parts[1].lower()
 
@@ -291,7 +342,7 @@ def get_resource_name(resource):
     return name
 
 
-def get_client(resource):
+def get_client(resource, func_config):
     resource_type = get_resource_type(resource)
     service = get_service_name(resource)
     resource_config = RESOURCE_TO_FUNCTION.get(resource_type)
@@ -301,7 +352,7 @@ def get_client(resource):
         # nothing to do for this resource
         return
     try:
-        if resource_config[ACTION_CREATE].get('boto_client') == 'resource':
+        if func_config.get('boto_client') == 'resource':
             return aws_stack.connect_to_resource(service)
         return aws_stack.connect_to_service(service)
     except Exception as e:
@@ -311,8 +362,12 @@ def get_client(resource):
 
 def describe_stack_resource(stack_name, logical_resource_id):
     client = aws_stack.connect_to_service('cloudformation')
-    result = client.describe_stack_resource(StackName=stack_name, LogicalResourceId=logical_resource_id)
-    return result['StackResourceDetail']
+    try:
+        result = client.describe_stack_resource(StackName=stack_name, LogicalResourceId=logical_resource_id)
+        return result['StackResourceDetail']
+    except Exception as e:
+        LOG.warning('Unable to get details for resource "%s" in CloudFormation stack "%s": %s' %
+                    (logical_resource_id, stack_name, e))
 
 
 def retrieve_resource_details(resource_id, resource_status, resources, stack_name):
@@ -327,7 +382,9 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             resource_id = resource_props['FunctionName'] if resource else resource_id
             return aws_stack.connect_to_service('lambda').get_function(FunctionName=resource_id)
         elif resource_type == 'Lambda::Version':
-            name = resource_props['FunctionName']
+            name = resource_props.get('FunctionName')
+            if not name:
+                return None
             func_name = aws_stack.lambda_function_name(name)
             func_version = name.split(':')[7] if len(name.split(':')) > 7 else '$LATEST'
             versions = aws_stack.connect_to_service('lambda').list_versions_by_function(FunctionName=func_name)
@@ -386,7 +443,12 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
             res_obj = aws_stack.connect_to_service('apigateway').get_resource(restApiId=api_id, resourceId=res_id)
             match = [v for (k, v) in res_obj['resourceMethods'].items()
                      if resource_props['HttpMethod'] in (v.get('httpMethod'), k)]
-            return match or None
+            int_props = resource_props.get('Integration')
+            if int_props:
+                match = [m for m in match if
+                    m.get('methodIntegration', {}).get('type') == int_props.get('Type') and
+                    m.get('methodIntegration', {}).get('httpMethod') == int_props.get('IntegrationHttpMethod')]
+            return any(match) or None
         elif resource_type == 'SQS::Queue':
             sqs_client = aws_stack.connect_to_service('sqs')
             queues = sqs_client.list_queues()
@@ -477,6 +539,8 @@ def resolve_ref(stack_name, ref, resources, attribute):
     resource_status = {}
     if stack_name:
         resource_status = describe_stack_resource(stack_name, ref)
+        if not resource_status:
+            return
         attr_value = resource_status.get(attribute)
         if attr_value not in [None, '']:
             return attr_value
@@ -496,25 +560,26 @@ def resolve_ref(stack_name, ref, resources, attribute):
 
 def resolve_refs_recursively(stack_name, value, resources):
     if isinstance(value, dict):
-        if len(value) == 1 and 'Ref' in value:
+        keys_list = list(value.keys())
+        # process special operators
+        if keys_list == ['Ref']:
             result = resolve_ref(stack_name, value['Ref'],
                 resources, attribute='PhysicalResourceId')
             return result
-        elif len(value) == 1 and 'Fn::GetAtt' in value:
-            return resolve_ref(stack_name, value['Fn::GetAtt'][0],
-                resources, attribute=value['Fn::GetAtt'][1])
-        else:
-            for key, val in iteritems(value):
-                value[key] = resolve_refs_recursively(stack_name, val, resources)
-        # process special operators
-        if len(value) == 1 and 'Fn::Join' in value:
-            return value['Fn::Join'][0].join(value['Fn::Join'][1])
-        if len(value) == 1 and 'Fn::Sub' in value:
-            result = value['Fn::Sub'][0]
-            for key, val in value['Fn::Sub'][1].items():
+        if keys_list and keys_list[0].lower() == 'fn::getatt':
+            return resolve_ref(stack_name, value[keys_list[0]][0],
+                resources, attribute=value[keys_list[0]][1])
+        if keys_list and keys_list[0].lower() == 'fn::join':
+            return value[keys_list[0]][0].join(value[keys_list[0]][1])
+        if keys_list and keys_list[0].lower() == 'fn::sub':
+            result = value[keys_list[0]][0]
+            for key, val in value[keys_list[0]][1].items():
                 val = resolve_refs_recursively(stack_name, val, resources)
                 result = result.replace('${%s}' % key, val)
             return result
+        else:
+            for key, val in iteritems(value):
+                value[key] = resolve_refs_recursively(stack_name, val, resources)
     if isinstance(value, list):
         for i in range(0, len(value)):
             value[i] = resolve_refs_recursively(stack_name, value[i], resources)
@@ -527,11 +592,15 @@ def update_resource(resource_id, resources, stack_name):
     if resource_type not in UPDATEABLE_RESOURCES:
         LOG.warning('Unable to update resource type "%s", id "%s"' % (resource_type, resource_id))
         return
+    LOG.info('Updating resource %s of type %s' % (resource_id, resource_type))
     props = resource['Properties']
     if resource_type == 'Lambda::Function':
         client = aws_stack.connect_to_service('lambda')
         keys = ('FunctionName', 'Role', 'Handler', 'Description', 'Timeout', 'MemorySize', 'Environment', 'Runtime')
         update_props = dict([(k, props[k]) for k in keys if k in props])
+        update_props = resolve_refs_recursively(stack_name, update_props, resources)
+        if 'Code' in props:
+            client.update_function_code(FunctionName=props['FunctionName'], **props['Code'])
         return client.update_function_configuration(**update_props)
     if resource_type == 'ApiGateway::Method':
         client = aws_stack.connect_to_service('apigateway')
@@ -577,11 +646,20 @@ def convert_data_types(func_details, params):
     return result
 
 
+def remove_none_values(params):
+    """ Remove None values recursively in the given object. """
+    def remove_nones(o, **kwargs):
+        if isinstance(o, dict):
+            for k, v in dict(o).items():
+                if v is None:
+                    o.pop(k)
+        return o
+    result = common.recurse_object(params, remove_nones)
+    return result
+
+
 def deploy_resource(resource_id, resources, stack_name):
     resource = resources[resource_id]
-    client = get_client(resource)
-    if not client:
-        return False
     resource_type = get_resource_type(resource)
     func_details = RESOURCE_TO_FUNCTION.get(resource_type)
     if not func_details:
@@ -590,8 +668,21 @@ def deploy_resource(resource_id, resources, stack_name):
 
     LOG.debug('Deploying resource type "%s" id "%s"' % (resource_type, resource_id))
     func_details = func_details[ACTION_CREATE]
+    func_details = func_details if isinstance(func_details, list) else [func_details]
+    results = []
+    for func in func_details:
+        client = get_client(resource, func)
+        if client:
+            result = deploy_resource_via_sdk_function(resource_id, resources, resource_type, func, stack_name)
+            results.append(result)
+    return (results or [None])[0]
+
+
+def deploy_resource_via_sdk_function(resource_id, resources, resource_type, func_details, stack_name):
+    resource = resources[resource_id]
+    client = get_client(resource, func_details)
     function = getattr(client, func_details['function'])
-    params = func_details['parameters']
+    params = func_details.get('parameters') or lambda_get_params()
     defaults = func_details.get('defaults', {})
     if 'Properties' not in resource:
         resource['Properties'] = {}
@@ -627,24 +718,28 @@ def deploy_resource(resource_id, resources, stack_name):
                     if prop_value is not None:
                         params[param_key] = prop_value
 
-    # convert refs and boolean strings
-    for param_key, prop_keys in dict(params).items():
-        tmp_value = params.get(param_key)
-        if tmp_value is not None:
-            params[param_key] = resolve_refs_recursively(stack_name, tmp_value, resources)
-        # Convert to boolean (TODO: do this recursively?)
-        if str(tmp_value).lower() in ['true', 'false']:
-            params[param_key] = str(tmp_value).lower() == 'true'
-
     # assign default value if empty
     params = common.merge_recursive(defaults, params)
+
+    # convert refs and boolean strings
+    for param_key, param_value in dict(params).items():
+        if param_value is not None:
+            param_value = params[param_key] = resolve_refs_recursively(stack_name, param_value, resources)
+        # Convert to boolean (TODO: do this recursively?)
+        if str(param_value).lower() in ['true', 'false']:
+            params[param_key] = str(param_value).lower() == 'true'
+
+    # convert any moto account IDs (123456789012) in ARNs to our format (000000000000)
+    params = json.loads(aws_stack.fix_account_id_in_arns(json.dumps(params)))
     # convert data types (e.g., boolean strings to bool)
     params = convert_data_types(func_details, params)
+    # remove None values, as they usually raise boto3 errors
+    params = remove_none_values(params)
 
     # invoke function
     try:
-        LOG.debug('Request for creating resource type "%s": %s %s' % (
-            resource_type, func_details['function'], params))
+        LOG.debug('Request for creating resource type "%s" in region %s: %s %s' % (
+            resource_type, aws_stack.get_region(), func_details['function'], params))
         result = function(**params)
     except Exception as e:
         LOG.warning('Error calling %s with params: %s for resource: %s' % (function, params, resource))
