@@ -318,6 +318,10 @@ def has_docker():
         return False
 
 
+def get_docker_container_names():
+    return bootstrap.get_docker_container_names()
+
+
 def is_port_open(port_or_url, http_path=None, expect_success=True, protocols=['tcp']):
     port = port_or_url
     host = 'localhost'
@@ -443,12 +447,12 @@ def recurse_object(obj, func, path=''):
     obj = func(obj, path=path)
     if isinstance(obj, list):
         for i in range(len(obj)):
-            path = '%s[%s]' % (path or '.', i)
-            obj[i] = recurse_object(obj[i], func, path)
+            tmp_path = '%s[%s]' % (path or '.', i)
+            obj[i] = recurse_object(obj[i], func, tmp_path)
     elif isinstance(obj, dict):
         for k, v in obj.items():
-            path = '%s%s' % ((path + '.') if path else '', k)
-            obj[k] = recurse_object(v, func, path)
+            tmp_path = '%s%s' % ((path + '.') if path else '', k)
+            obj[k] = recurse_object(v, func, tmp_path)
     return obj
 
 
@@ -596,6 +600,8 @@ def is_linux():
 
 def is_alpine():
     try:
+        if not os.path.exists('cat /etc/issue'):
+            return False
         out = to_str(subprocess.check_output('cat /etc/issue', shell=True))
         return 'Alpine' in out
     except subprocess.CalledProcessError:
