@@ -26,6 +26,10 @@ def create_lambda_archive(script, get_content=False, libs=[], runtime=None, file
     script_file = os.path.join(tmp_dir, file_name)
     if os.path.sep in script_file:
         mkdir(os.path.dirname(script_file))
+        # create __init__.py files along the path to allow Python imports
+        path = file_name.split(os.path.sep)
+        for i in range(1, len(path)):
+            save_file(os.path.join(tmp_dir, *(path[:i] + ['__init__.py'])), '')
     save_file(script_file, script)
     # copy libs
     for lib in libs:
